﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using PuferFishContracts.DataModels;
using PuferFishContracts.Enums;
using PuferFishContracts.Exceptions;
using PuferFishDataBase.Implementations;
using PuferFishDataBase.Models;

namespace PuferFishTests.StoragesContracts;

[TestFixture]
internal class ProductStorageContractTests : BaseStorageContractTest
{
    private ProductStorageContract _productStorageContract;
    [SetUp]
    public void SetUp()
    {
        _productStorageContract = new
        ProductStorageContract(PuferFishDbContext);
    }
    [TearDown]
    public void TearDown()
    {
        PuferFishDbContext.Database.ExecuteSqlRaw("TRUNCATE \"Products\"CASCADE; ");
    }
    [Test]
    public void Try_GetList_WhenHaveRecords_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 1");
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 2");
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 3");
        var list = _productStorageContract.GetList();
        Assert.That(list, Is.Not.Null);
        Assert.That(list, Has.Count.EqualTo(3));
        AssertElement(list.First(x => x.Id == product.Id), product);
    }
    [Test]
    public void Try_GetList_WhenNoRecords_Test()
    {
        var list = _productStorageContract.GetList();
        Assert.That(list, Is.Not.Null);
        Assert.That(list, Is.Empty);
    }
    [Test]
    public void Try_GetList_OnlyActual_Test()
    {
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 1", isDeleted: true);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 2", isDeleted: false);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 3", isDeleted: false);
        var list = _productStorageContract.GetList(onlyActive: true);
        Assert.That(list, Is.Not.Null);
        Assert.Multiple(() =>
        {
            Assert.That(list, Has.Count.EqualTo(2));
            Assert.That(!list.Any(x => x.IsDeleted));
        });
    }
    [Test]
    public void Try_GetList_IncludeNoActual_Test()
    {
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 1", isDeleted: true);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 2", isDeleted: true);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 3", isDeleted: false);
        var list = _productStorageContract.GetList(onlyActive: false);
        Assert.That(list, Is.Not.Null);
        Assert.Multiple(() =>
        {
            Assert.That(list, Has.Count.EqualTo(3));
            Assert.That(list.Count(x => x.IsDeleted), Is.EqualTo(2));
            Assert.That(list.Count(x => !x.IsDeleted), Is.EqualTo(1));
        });
    }
    
    [Test]
    public void Try_GetHistoryByProductId_WhenHaveRecords_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 1");
        InsertProductHistoryToDatabaseAndReturn(product.Id, 20, DateTime.UtcNow.AddDays(-1));
        InsertProductHistoryToDatabaseAndReturn(product.Id, 30, DateTime.UtcNow.AddMinutes(-10));
        InsertProductHistoryToDatabaseAndReturn(product.Id, 40, DateTime.UtcNow.AddDays(1));
        var list = _productStorageContract.GetHistoryByProductId(product.Id);
        Assert.That(list, Is.Not.Null);
        Assert.That(list, Has.Count.EqualTo(3));
    }
    [Test]
    public void Try_GetHistoryByProductId_WhenNoRecords_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), "name 1");
        InsertProductHistoryToDatabaseAndReturn(product.Id, 20, DateTime.UtcNow.AddDays(-1));
        InsertProductHistoryToDatabaseAndReturn(product.Id, 30, DateTime.UtcNow.AddMinutes(-10));
        InsertProductHistoryToDatabaseAndReturn(product.Id, 40, DateTime.UtcNow.AddDays(1));
        var list = _productStorageContract.GetHistoryByProductId(Guid.NewGuid().ToString());
        Assert.That(list, Is.Not.Null);
        Assert.That(list, Has.Count.EqualTo(0));
    }
    [Test]
    public void Try_GetElementById_WhenHaveRecord_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString());
        AssertElement(_productStorageContract.GetElementById(product.Id),
        product);
    }
    [Test]
    public void Try_GetElementById_WhenNoRecord_Test()
    {
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString());
        Assert.That(() => _productStorageContract.GetElementById(Guid.NewGuid().ToString()), Is.Null);
    }
    [Test]
    public void Try_GetElementById_WhenRecordHasDeleted_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), isDeleted: true);
        Assert.That(() => _productStorageContract.GetElementById(product.Id),
        Is.Null);
    }
    [Test]
    public void Try_GetElementByName_WhenHaveRecord_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString());
        AssertElement(_productStorageContract.GetElementByName(product.ProductName), product);
    }
    [Test]
    public void Try_GetElementByName_WhenNoRecord_Test()
    {
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString());
        Assert.That(() => _productStorageContract.GetElementByName("name"),
        Is.Null);
    }
    [Test]
    public void Try_GetElementByName_WhenRecordHasDeleted_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), isDeleted: true);
        Assert.That(() => _productStorageContract.GetElementById(product.ProductName), Is.Null);
    }
    [Test]
    public void Try_AddElement_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), isDeleted: false);
        _productStorageContract.AddElement(product);
        AssertElement(GetProductFromDatabaseById(product.Id), product);
    }
    [Test]
    public void Try_AddElement_WhenIsDeletedIsTrue_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), isDeleted: true);
        Assert.That(() => _productStorageContract.AddElement(product),
        Throws.Nothing);
        AssertElement(GetProductFromDatabaseById(product.Id),
        CreateModel(product.Id, isDeleted: false));
    }
    [Test]
    public void Try_AddElement_WhenHaveRecordWithSameId_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString());
        InsertProductToDatabaseAndReturn(product.Id, productName: "name unique");
        Assert.That(() => _productStorageContract.AddElement(product),
        Throws.TypeOf<ElementExistsException>());
    }
    [Test]
    public void Try_AddElement_WhenHaveRecordWithSameName_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), "name unique", isDeleted: false);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), productName: product.ProductName, isDeleted: false);
        Assert.That(() => _productStorageContract.AddElement(product),
        Throws.TypeOf<ElementExistsException>());
    }
    [Test]
    public void Try_AddElement_WhenHaveRecordWithSameNameButOneWasDeleted_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), "name unique", isDeleted: false);
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), productName: product.ProductName, isDeleted: true);
        Assert.That(() => _productStorageContract.AddElement(product),
        Throws.Nothing);
    }
    [Test]
    public void Try_UpdElement_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), isDeleted: false);
        InsertProductToDatabaseAndReturn(product.Id, isDeleted: false);
        _productStorageContract.UpdElement(product);
        AssertElement(GetProductFromDatabaseById(product.Id), product);
    }
    [Test]
    public void Try_UpdElement_WhenIsDeletedIsTrue_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), isDeleted: true);
        InsertProductToDatabaseAndReturn(product.Id, isDeleted: false);
        _productStorageContract.UpdElement(product);
        AssertElement(GetProductFromDatabaseById(product.Id),
        CreateModel(product.Id, isDeleted: false));
    }
    [Test]
    public void Try_UpdElement_WhenNoRecordWithThisId_Test()
    {
        Assert.That(() =>
        _productStorageContract.UpdElement(CreateModel(Guid.NewGuid().ToString())), Throws.TypeOf<ElementNotFoundException>());
    }
    [Test]
    public void Try_UpdElement_WhenHaveRecordWithSameName_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), "name unique", isDeleted: false);
        InsertProductToDatabaseAndReturn(product.Id, productName: "name");
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), productName: product.ProductName);
        Assert.That(() => _productStorageContract.UpdElement(product),
        Throws.TypeOf<ElementExistsException>());
    }
    [Test]
    public void
    Try_UpdElement_WhenHaveRecordWithSameNameButOneWasDeleted_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString(), "name unique", isDeleted: false);
        InsertProductToDatabaseAndReturn(product.Id, productName: "name");
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), productName: product.ProductName, isDeleted: true);
        Assert.That(() => _productStorageContract.UpdElement(product),
        Throws.Nothing);
    }
    [Test]
    public void Try_UpdElement_WhenRecordWasDeleted_Test()
    {
        var product = CreateModel(Guid.NewGuid().ToString());
        InsertProductToDatabaseAndReturn(product.Id, isDeleted: true);
        Assert.That(() => _productStorageContract.UpdElement(product),
        Throws.TypeOf<ElementNotFoundException>());
    }
    [Test]
    public void Try_DelElement_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), isDeleted: false);
        _productStorageContract.DelElement(product.Id);
        var element = GetProductFromDatabaseById(product.Id);
        Assert.Multiple(() =>
        {
            Assert.That(element, Is.Not.Null);
            Assert.That(element!.IsDeleted);
        });
    }
    [Test]
    public void Try_DelElement_WhenNoRecordWithThisId_Test()
    {
        Assert.That(() =>
        _productStorageContract.DelElement(Guid.NewGuid().ToString()),
        Throws.TypeOf<ElementNotFoundException>());
    }
    [Test]
    public void Try_DelElement_WhenRecordWasDeleted_Test()
    {
        var product =
        InsertProductToDatabaseAndReturn(Guid.NewGuid().ToString(), isDeleted: true);
        Assert.That(() => _productStorageContract.DelElement(product.Id),
        Throws.TypeOf<ElementNotFoundException>());
    }
   
    private Product InsertProductToDatabaseAndReturn(string id, string productName = "test", ProductType productType =
    ProductType.Onigiri, double price = 1, bool isDeleted = false)
    {
        var product = new Product()
        {
            Id = id,
            ProductName = productName,
            ProductType = productType,
            Price = price,
            IsDeleted = isDeleted
        };
        PuferFishDbContext.Products.Add(product);
        PuferFishDbContext.SaveChanges();
        return product;
    }
    private ProductHistory InsertProductHistoryToDatabaseAndReturn(string productId, double price, DateTime changeDate)
    {
        var productHistory = new ProductHistory()
        {
            Id = Guid.NewGuid().ToString(),
            ProductId = productId,
            OldPrice = price,
            ChangeDate = changeDate
        };
        PuferFishDbContext.ProductHistories.Add(productHistory);
        PuferFishDbContext.SaveChanges();
        return productHistory;
    }
    private static void AssertElement(ProductDataModel? actual, Product expected)
    {
        Assert.That(actual, Is.Not.Null);
        Assert.Multiple(() =>
        {
            Assert.That(actual.Id, Is.EqualTo(expected.Id));
            Assert.That(actual.ProductName,
            Is.EqualTo(expected.ProductName));
            Assert.That(actual.ProductType,
            Is.EqualTo(expected.ProductType));
            Assert.That(actual.Price, Is.EqualTo(expected.Price));
            Assert.That(actual.IsDeleted, Is.EqualTo(expected.IsDeleted));
        });
    }
    private static ProductDataModel CreateModel(string id, string productName = "test", ProductType productType =
    ProductType.Onigiri, double price = 1, bool isDeleted = false)
    => new(id, productName, productType, price, isDeleted);
    private Product? GetProductFromDatabaseById(string id) => PuferFishDbContext.Products.FirstOrDefault(x => x.Id == id);
    private static void AssertElement(Product? actual, ProductDataModel expected)
    {
        Assert.That(actual, Is.Not.Null);
        Assert.Multiple(() =>
        {
            Assert.That(actual.Id, Is.EqualTo(expected.Id));
            Assert.That(actual.ProductName,
            Is.EqualTo(expected.ProductName));
            Assert.That(actual.ProductType,
            Is.EqualTo(expected.ProductType));
            Assert.That(actual.Price, Is.EqualTo(expected.Price));
            Assert.That(actual.IsDeleted, Is.EqualTo(expected.IsDeleted));
        });
    }
}
